﻿#include <QDebug>
#include <QtXml/qdom.h>
#include <QFile>
#include "../common/sources/system/cgraphics2d.h"
#include "../common/sources/system/filecache.h"
#include "interface.h"
#include "game.h"
#include "player.h"
#include "keyboard.h"
#include "objects.h"

CInterface   interface;

static const int   championsColors[4][3] =
{
    {  0, 222,   0},
    {255, 255,   0},
    {255,   0,   0},
    {  0,   0, 255}
};

//------------------------------------------------------------
CInterface::CInterface()
{
    mainState = eMainGame;
    weaponsAreaState = eWeaponsAreaWeapons;
    currentChampion = -1;
    currentWeapon = -1;
    isResurrecting = false;
    isPressingEye = false;
    arrowHighlightTime = 0;
    selectedChampion = 0;
    delayedArrowTime = 0;

    for (int i = 0; i < 4; ++i)
    {
        weaponCoolDown[i] = new QTimer();
        weaponCoolDown[i]->setSingleShot(true);
    }

    arrowsRects[eArrowTurnLeft]  = CRect(CVec2(234, 125), CVec2(261, 145));
    arrowsRects[eArrowForward]   = CRect(CVec2(263, 125), CVec2(289, 145));
    arrowsRects[eArrowTurnRight] = CRect(CVec2(291, 125), CVec2(318, 145));

    arrowsRects[eArrowLeft]      = CRect(CVec2(234, 147), CVec2(261, 167));
    arrowsRects[eArrowBackward]  = CRect(CVec2(263, 147), CVec2(289, 167));
    arrowsRects[eArrowRight]     = CRect(CVec2(291, 147), CVec2(318, 167));

    // initialise les positions des objets dans la feuille de personnage
    bodyInfos[CCharacter::eBodyPartHead].pos      = CVec2(34,  59);
    bodyInfos[CCharacter::eBodyPartNeck].pos      = CVec2( 6,  66);
    bodyInfos[CCharacter::eBodyPartTorso].pos     = CVec2(34,  79);
    bodyInfos[CCharacter::eBodyPartLeftHand].pos  = CVec2( 6,  86);
    bodyInfos[CCharacter::eBodyPartRightHand].pos = CVec2(62,  86);
    bodyInfos[CCharacter::eBodyPartLegs].pos      = CVec2(34,  99);
    bodyInfos[CCharacter::eBodyPartFeet].pos      = CVec2(34, 119);
    bodyInfos[CCharacter::eBodyPartPouch1].pos    = CVec2( 6, 106);
    bodyInfos[CCharacter::eBodyPartPouch2].pos    = CVec2( 6, 123);
    bodyInfos[CCharacter::eBodyPartQuiver1].pos   = CVec2(62, 106);
    bodyInfos[CCharacter::eBodyPartQuiver2].pos   = CVec2(79, 106);
    bodyInfos[CCharacter::eBodyPartQuiver3].pos   = CVec2(62, 123);
    bodyInfos[CCharacter::eBodyPartQuiver4].pos   = CVec2(79, 123);

    bodyInfos[CCharacter::eBodyPartHead].type      = 'H';
    bodyInfos[CCharacter::eBodyPartNeck].type      = 'N';
    bodyInfos[CCharacter::eBodyPartTorso].type     = 'T';
    bodyInfos[CCharacter::eBodyPartLeftHand].type  = 'h';
    bodyInfos[CCharacter::eBodyPartRightHand].type = 'h';
    bodyInfos[CCharacter::eBodyPartLegs].type      = 'L';
    bodyInfos[CCharacter::eBodyPartFeet].type      = 'F';
    bodyInfos[CCharacter::eBodyPartPouch1].type    = 'P';
    bodyInfos[CCharacter::eBodyPartPouch2].type    = 'P';
    bodyInfos[CCharacter::eBodyPartQuiver1].type   = 'Q';
    bodyInfos[CCharacter::eBodyPartQuiver2].type   = 'q';
    bodyInfos[CCharacter::eBodyPartQuiver3].type   = 'q';
    bodyInfos[CCharacter::eBodyPartQuiver4].type   = 'q';

    for (int i = 0; i < BACKPACK_SIZE; ++i)
    {
        int x = 66 + ((i + 1) % 9) * 17;
        int y = 49 + ((i + 1) / 9) * 17;
        bodyInfos[CCharacter::eBodyPartBackpack + i].pos = CVec2(x, y);
        bodyInfos[CCharacter::eBodyPartBackpack + i].type = 'B';
    }

    // images par défaut pour les parties du corps
    for (int i = 0; i < CCharacter::eBodyPartCount; ++i)
        bodyInfos[i].defaultImage = -1;

    bodyInfos[CCharacter::eBodyPartHead].defaultImage         = 24;
    bodyInfos[CCharacter::eBodyPartNeck].defaultImage         = 16;
    bodyInfos[CCharacter::eBodyPartTorso].defaultImage        = 26;
    bodyInfos[CCharacter::eBodyPartLeftHand].defaultImage     = 20;
    bodyInfos[CCharacter::eBodyPartRightHand].defaultImage    = 22;
    bodyInfos[CCharacter::eBodyPartLegs].defaultImage         = 28;
    bodyInfos[CCharacter::eBodyPartFeet].defaultImage         = 30;
    bodyInfos[CCharacter::eBodyPartPouch1].defaultImage       = 17;
    bodyInfos[CCharacter::eBodyPartQuiver1].defaultImage      = 18;
    bodyInfos[CCharacter::eBodyPartBackpack + 8].defaultImage = 19;
}

//------------------------------------------------------------
CInterface::~CInterface()
{
    for (int i = 0; i < 4; ++i)
        delete weaponCoolDown[i];
}

//------------------------------------------------------------
void    CInterface::display(QImage* image)
{
    if (isInInventory() == true)
        drawInventory(image);

    for (int i = 0; i < 4; ++i)
        drawChampion(image, i);

    drawObjectName(image);
    drawSpells(image);
    drawWeaponsArea(image);
    drawArrows(image);
}

//------------------------------------------------------------
bool    CInterface::isInInventory()
{
    return (mainState == eMainInventory);
}

//------------------------------------------------------------
bool    CInterface::isChampionEmpty(int num)
{
    CCharacter* c = &game.characters[num];
    return c->portrait == -1;
}

//------------------------------------------------------------
bool    CInterface::isChampionDead(int num)
{
    CCharacter* c = &game.characters[num];
    return c->isDead();
}

//------------------------------------------------------------
bool    CInterface::isWeaponEmpty(int num)
{
    return (isChampionDead(num) || isChampionEmpty(num));
}

//------------------------------------------------------------
bool    CInterface::isWeaponGreyed(int num)
{
    return (weaponCoolDown[num]->isActive() || isResurrecting);
}

//------------------------------------------------------------
bool    CInterface::isArrowsGreyed()
{
    return isInInventory();
}

//------------------------------------------------------------
bool    CInterface::isSpellsGreyed()
{
    return isResurrecting;
}

//------------------------------------------------------------
void    CInterface::drawBar(QImage* image, int num, CVec2 pos, CCharacter::EStatsNames statName)
{
    CCharacter::SStat   stat = getStat(num, statName);

    if  (stat.maxValue == 0)
        return;

    pos.y = 27 - (25 * stat.value) / stat.maxValue;
    CRect   rect(pos, CVec2(pos.x + 3, 26));
    QColor color(championsColors[num][0],
                 championsColors[num][1],
                 championsColors[num][2]);
    graph2D.rect(image, rect, color, true);
}

//------------------------------------------------------------
void    CInterface::drawChampionName(QImage* image, int num, CVec2 pos, bool isFullName)
{
    CCharacter* c = &game.characters[num];
    std::string name = c->firstName;

    if (isFullName == true)
        name += " " + c->lastName;

    QColor  color = (num == selectedChampion ? HIGHLIGHT_TEXT_COLOR : DEFAULT_TEXT_COLOR);
    drawText(image, pos, eFontStandard, name.c_str(), color);
}

//------------------------------------------------------------
void    CInterface::drawChampion(QImage* image, int num)
{
    if (isChampionEmpty(num) == false)
    {
        CVec2   basePos(69 * num, 0);
        CVec2   namePos = basePos + CVec2(1, 1);
        CRect   handMouseRect(basePos + CVec2(0, 7),  basePos + CVec2(42, 28));
        CRect   barsMouseRect(basePos + CVec2(43, 0), basePos + CVec2(66, 28));

        if (isChampionDead(num) == true)
        {
            QImage  champBg = fileCache.getImage("gfx/interface/DeadChampInfo.png");
            graph2D.drawImage(image, basePos, champBg);

            // écrit le nom
            drawChampionName(image, num, namePos, false);
        }
        else
        {
            QImage  champBg = fileCache.getImage("gfx/interface/ChampInfo.png");
            graph2D.drawImage(image, basePos, champBg);

            if (isInInventory() == true && currentChampion == num)
            {
                // dessine un rectangle dans le fond
                CRect   rect2(basePos, basePos + CVec2(42, 6));
                graph2D.rect(image, rect2, INVENTORY_BG_COLOR, true);

                CVec2   headPos = basePos + CVec2(7, 0);
                CVec2   headSize = CVec2(CHAMP_PORTRAIT_WIDTH - 1, CHAMP_PORTRAIT_HEIGHT - 1);
                CRect   headRect(headPos, headPos + headSize);
                graph2D.rect(image, headRect, INVENTORY_SLOT_COLOR, true);

                // dessine la tête
                CCharacter* c = &game.characters[num];
                CRect   imageRect = getChampionPortraitRect(c->portrait);
                QImage  portraits = fileCache.getImage("gfx/interface/ChampPortraits.png");
                graph2D.drawImageAtlas(image, headPos, portraits, imageRect);

                // zone souris
                if (isResurrecting == false)
                    mouse.addArea(eMouseArea_3DView, barsMouseRect, eCursor_Hand);
            }
            else
            {
                // dessine les rectangles
                CRect   rect(basePos + CVec2(3, 9),
                             basePos + CVec2(20, 26));
                graph2D.rect(image, rect, INVENTORY_SLOT_COLOR, false);

                CRect   rect2(basePos + CVec2(23, 9),
                              basePos + CVec2(40, 26));
                graph2D.rect(image, rect2, INVENTORY_SLOT_COLOR, false);

                // dessine les mains
                mouse.addArea(eMouseArea_None, handMouseRect, eCursor_Hand);
                drawBodyPart(image, basePos + CVec2(4, 10), num, CCharacter::eBodyPartLeftHand, !isResurrecting);
                drawBodyPart(image, basePos + CVec2(24, 10), num, CCharacter::eBodyPartRightHand, !isResurrecting);

                // écrit le nom
                drawChampionName(image, num, namePos, false);

                // zone souris
                if (isResurrecting == false)
                    mouse.addArea(eMouseArea_Inventory, barsMouseRect, eCursor_Hand, (void*)num);

                CRect nameRect(basePos, basePos + CVec2(42, 6));
                mouse.addArea(eMouseArea_SelectChamp, nameRect, eCursor_Arrow, (void*)num);
            }

            // barres
            drawBar(image, num, basePos + CVec2(46, 2), CCharacter::eStatHealth);
            drawBar(image, num, basePos + CVec2(53, 2), CCharacter::eStatStamina);
            drawBar(image, num, basePos + CVec2(60, 2), CCharacter::eStatMana);
        }
    }
}

//------------------------------------------------------------
void CInterface::drawObjectName(QImage* image)
{
    int type = mouse.mObjectInHand.getType();

    if (type != 0)
    {
        std::string& name = objects.mObjectInfos[type - 1].name;
        drawText(image, CVec2(233, 33), eFontStandard, name.c_str(), MAIN_INTERFACE_COLOR);
    }
}

//------------------------------------------------------------
void    CInterface::drawSpells(QImage* image)
{
    QImage  spellsBg = fileCache.getImage("gfx/interface/SpellsArea.png");
    CVec2   pos(233, 50);
    graph2D.drawImage(image, pos, spellsBg);

    for (int i = 0; i < 6; ++i)
    {
        CVec2 pos(239 + i * 14, 54);
        char word[2] = "a";
        word[0] += i;
        drawText(image, pos, eFontStandard, word, MAIN_INTERFACE_COLOR);
    }

    if (isSpellsGreyed() == true)
    {
        CRect   rect(CVec2(233, 42),
                     CVec2(319, 74));
        graph2D.patternRectangle(image, rect);
    }
}

//------------------------------------------------------------
void    CInterface::drawWeaponsArea(QImage* image)
{
    if (weaponsAreaState == eWeaponsAreaWeapons)
    {
        for (int i = 0; i < 4; ++i)
            drawWeapon(image, i);
    }
    else if (weaponsAreaState == eWeaponsAreaAttacks)
    {
        QImage  spellsBg = fileCache.getImage("gfx/interface/WeaponsActions.png");
        CVec2   pos(233, 77);
        graph2D.drawImage(image, pos, spellsBg);
    }
    else
    {
        QImage  spellsBg = fileCache.getImage("gfx/interface/DamageDone.png");
        CVec2   pos(232, 77);
        graph2D.drawImage(image, pos, spellsBg);
    }
}

//------------------------------------------------------------
void    CInterface::drawWeapon(QImage* image, int num)
{
    if (isWeaponEmpty(num) == false)
    {
        CRect   rect(CVec2(233 + 22 * num, 86),
                     CVec2(252 + 22 * num, 120));
        graph2D.rect(image, rect, MAIN_INTERFACE_COLOR, true);

        if (isWeaponGreyed(num) == true)
            graph2D.patternRectangle(image, rect);
    }
}

//------------------------------------------------------------
void    CInterface::drawArrows(QImage* image)
{
    QImage  arrowsBg = fileCache.getImage("gfx/interface/MovementArrows.png");
    CVec2   pos(233, 124);
    graph2D.drawImage(image, pos, arrowsBg);

    if (isArrowsGreyed() == true)
    {
        CRect   rect(CVec2(233, 124),
                     CVec2(319, 168));
        graph2D.patternRectangle(image, rect);
    }
    else
    {
        static const EMouseAreaTypes arrowsMouseAreas[] =
        {
            eMouseArea_ArrowForward,
            eMouseArea_ArrowBackward,
            eMouseArea_ArrowLeft,
            eMouseArea_ArrowRight,
            eMouseArea_ArrowTurnLeft,
            eMouseArea_ArrowTurnRight
        };

        for (int i = 0; i < eArrowsCount; ++i)
        {
            mouse.addArea(arrowsMouseAreas[i], arrowsRects[i]);
        }

        if (arrowHighlightTime > 0)
        {
            arrowHighlightTime--;
            graph2D.Xor(image, arrowsRects[currentArrow], MAIN_INTERFACE_COLOR);
        }
    }
}

//------------------------------------------------------------
void    CInterface::displayStat(QImage* image, CVec2 pos, int x2, CCharacter::EStatsNames num)
{
    static char temp[256];

    sprintf(temp, "STATS%02d", num);
    drawText(image, pos, eFontStandard, getText(temp).c_str(), DEFAULT_TEXT_COLOR);

    CCharacter::SStat stat = getStat(currentChampion, num);
    sprintf(temp, "%3d/%3d", stat.value, stat.maxValue);
    pos.x = x2;
    drawText(image, pos, eFontStandard, temp, DEFAULT_TEXT_COLOR);
}

//------------------------------------------------------------
void    CInterface::drawInfos(QImage* image)
{
    // dessine l'image des infos
    QImage  infoBg = fileCache.getImage("gfx/interface/EmptyInfo.png");
    graph2D.drawImage(image, CVec2(80, 85), infoBg);

    int objType = mouse.mObjectInHand.getType();

    if (objType == 0)
    {
        // infos du champion
        displayStat(image, CVec2(108, 115), 174, CCharacter::eStatStrength);
        displayStat(image, CVec2(108, 122), 174, CCharacter::eStatDexterity);
        displayStat(image, CVec2(108, 129), 174, CCharacter::eStatWisdom);
        displayStat(image, CVec2(108, 136), 174, CCharacter::eStatVitality);
        displayStat(image, CVec2(108, 143), 174, CCharacter::eStatAntiMagic);
        displayStat(image, CVec2(108, 150), 174, CCharacter::eStatAntiFire);
    }
    else
    {
        // infos d'un objet
        QImage  eye = fileCache.getImage("gfx/interface/Eye.png");
        graph2D.drawImage(image, CVec2(83, 90), eye);

        QImage  circle = fileCache.getImage("gfx/interface/Circle.png");
        graph2D.drawImage(image, CVec2(106, 87), circle);

        CObjects::CObjectInfo&  infos = objects.mObjectInfos[objType - 1];
        QImage  objSheet = fileCache.getImage(infos.imageFile.toLocal8Bit().constData());
        CRect   objRect = interface.getItemRect(infos.imageNum);
        graph2D.drawImageAtlas(image, CVec2(111, 92), objSheet, objRect);

        // nom de l'objet
        drawText(image, CVec2(134, 98), eFontStandard, infos.name.c_str(), DEFAULT_TEXT_COLOR);

        int y = 116;

        if (objType == 144)
        {
            // gourde pleine
            drawText(image, CVec2(108, y), eFontStandard, getText("OBJ_INFO00").c_str(), DEFAULT_TEXT_COLOR);
            y += 7;
        }
        else if (objType == 145)
        {
            // gourde vide
            drawText(image, CVec2(108, y), eFontStandard, getText("OBJ_INFO03").c_str(), DEFAULT_TEXT_COLOR);
            y += 7;
        }
        else if (infos.positions.contains("c"))
        {
            // comestible
            drawText(image, CVec2(108, y), eFontStandard, getText("OBJ_INFO04").c_str(), DEFAULT_TEXT_COLOR);
            y += 7;
        }

        // poids
        static char temp[256];
        sprintf(temp, "%s %.1f KG.", getText("OBJ_INFO05").c_str(), (float)infos.weight / 10.0f);
        drawText(image, CVec2(108, y), eFontStandard, temp, DEFAULT_TEXT_COLOR);
    }
}

//------------------------------------------------------------
void    CInterface::drawInventory(QImage* image)
{
    CRect   mainRect(CVec2(0, 33), CVec2(MAINVIEW_WIDTH - 1, 33 + MAINVIEW_HEIGHT - 1));
    mouse.addArea(eMouseArea_None, mainRect, eCursor_Hand);

    QImage  bodyParts = fileCache.getImage("gfx/interface/Items6.png");

    // dessine le fond
    QImage  inventoryBg = fileCache.getImage("gfx/interface/InventoryMain.png");
    graph2D.drawImage(image, CVec2(0, 33), inventoryBg);

    // écrit le nom
    drawChampionName(image, currentChampion, CVec2(3, 36), true);

    // affiche santé/vigueur/mana
    displayStat(image, CVec2(5, 145), 55, CCharacter::eStatHealth);
    displayStat(image, CVec2(5, 153), 55, CCharacter::eStatStamina);
    displayStat(image, CVec2(5, 161), 55, CCharacter::eStatMana);

    // affiche la charge
    CCharacter* c = &game.characters[currentChampion];
    QColor  loadColor;
    if (c->overload() == 0)
        loadColor = DEFAULT_TEXT_COLOR;
    else if (c->overload() == 1)
        loadColor = OVERLOAD1_COLOR;
    else
        loadColor = OVERLOAD2_COLOR;

    drawText(image, CVec2(104, 161), eFontStandard, getText("STATS09").c_str(), loadColor);
    static char temp[256];
    CCharacter::SStat stat = getStat(currentChampion, CCharacter::eStatLoad);
    sprintf(temp, "%5.1f/%3d KG", (float)stat.value / 10.0f, stat.maxValue / 10);

    drawText(image, CVec2(148, 161), eFontStandard, temp, loadColor);

    // l'oeil
    CRect   eyePosRect(CVec2(12, 46), CVec2(27, 61));
    mouse.addArea(eMouseArea_Eye, eyePosRect, (mouse.mButtonPressed == false ? eCursor_Hand : eCursor_None));

    if (isPressingEye == true)
    {
        CRect   imageRect = getItemRect(11);
        graph2D.drawImageAtlas(image, eyePosRect.tl, bodyParts, imageRect);
    }

    // affiche les objets
    for (int i = 0; i < CCharacter::eBodyPartCount; ++i)
        drawBodyPart(image, bodyInfos[i].pos, currentChampion, (CCharacter::EBodyParts)i);

    // boutons
    if (isResurrecting == true)
    {
        // cache les boutons sauver/dormir/fermer
        CRect   rect(CVec2(174, 35), CVec2(218, 45));
        graph2D.rect(image, rect, INVENTORY_BG_COLOR, true);
    }
    else
    {
        // bouton fermer
        CRect   closeRect(CVec2(210, 36), CVec2(218, 44));
        mouse.addArea(eMouseArea_3DView, closeRect, eCursor_Hand);
    }

    if (isPressingEye == true)
    {
        drawInfos(image);
    }
    else if (isResurrecting == true)
    {
        // affiche l'image ressusciter/réincarner
        QImage  resurrectImage = fileCache.getImage("gfx/interface/ResurrectReincarnate.png");
        CVec2   pos(80, 85);
        graph2D.drawImage(image, pos, resurrectImage);

        if (mouse.mObjectInHand.getType() == 0)
        {
			// zone souris ressusciter
            CRect   resurrectRect(CVec2(103, 85), CVec2(159, 143));
            mouse.addArea(eMouseArea_Resurrect, resurrectRect, eCursor_Hand);

	        // zone souris annuler
            CRect   cancelRect(CVec2(103, 145), CVec2(218, 157));
            mouse.addArea(eMouseArea_3DView, cancelRect, eCursor_Hand);
        }
    }
}

//------------------------------------------------------------
CObject& CInterface::getWeapon(int num)
{
    CCharacter* c = &game.characters[num];
    return c->bodyObjects[CCharacter::eBodyPartRightHand];
}

//------------------------------------------------------------
CCharacter::SStat   CInterface::getStat(int num, CCharacter::EStatsNames statName)
{
    CCharacter* c = &game.characters[num];
    return c->stats[statName];
}

//------------------------------------------------------------
CRect   CInterface::getChampionPortraitRect(int num)
{
    int x = num % 8;
    int y = num / 8;
    CRect   rect;

    rect.tl.x = x * CHAMP_PORTRAIT_WIDTH;
    rect.tl.y = y * CHAMP_PORTRAIT_HEIGHT;
    rect.br.x = rect.tl.x + CHAMP_PORTRAIT_WIDTH - 1;
    rect.br.y = rect.tl.y + CHAMP_PORTRAIT_HEIGHT - 1;

    return rect;
}

//------------------------------------------------------------
CRect   CInterface::getItemRect(int num)
{
    int x = num % 16;
    int y = num / 16;
    CRect   rect;

    rect.tl.x = x * ITEM_WIDTH;
    rect.tl.y = y * ITEM_HEIGHT;
    rect.br.x = rect.tl.x + ITEM_WIDTH - 1;
    rect.br.y = rect.tl.y + ITEM_HEIGHT - 1;

    return rect;
}

//------------------------------------------------------------
void    CInterface::drawBodyPart(QImage* image, CVec2 pos, int championNum, CCharacter::EBodyParts part, bool enableArea)
{
    CCharacter* c = &game.characters[championNum];
    int     objType = c->bodyObjects[part].getType();

    if (objType != 0)
    {
        // dessine l'objet dans ce slot
        CObjects::CObjectInfo  object = objects.mObjectInfos[objType - 1];
        QImage  objImage = fileCache.getImage(object.imageFile.toLocal8Bit().constData());
        CRect   rect = getItemRect(object.imageNum);
        graph2D.drawImageAtlas(image, pos, objImage, rect);
    }
    else if (bodyInfos[part].defaultImage != -1)
    {
        // dessine l'image par défaut
        QImage  bodyParts = fileCache.getImage("gfx/interface/Items6.png");
        CRect   rect = getItemRect(bodyInfos[part].defaultImage);
        graph2D.drawImageAtlas(image, pos, bodyParts, rect);
    }

    // zone souris
    int     objHandType = mouse.mObjectInHand.getType();
    CRect   mouseRect(pos, pos + CVec2(ITEM_WIDTH - 1, ITEM_HEIGHT - 1));
    bool    isPositionCompatible = true;

    // Teste si l'objet qu'on tient peut être mis dans ce type de slot
    if (objHandType != 0)
    {
        QString&    positions = objects.mObjectInfos[objHandType - 1].positions;
        isPositionCompatible = positions.contains(bodyInfos[part].type);
    }

    if (isPositionCompatible == true && enableArea == true)
        mouse.addArea(eMouseArea_InvObject, mouseRect, eCursor_Hand, (void*)championNum, (void*)part);
}

//------------------------------------------------------------
void   CInterface::drawText(QImage* image, CVec2 pos, EFonts font, const char* text, QColor color)
{
    // Table des numéros de sprites pour chaque caractère ASCII du caractère 32 au caractère 120
    // Les lettres en minuscule sont utilisées pour les sorts
    //  !"#$%&'()*+,-./
    // 0123456789:;<=>?
    // @ABCDEFGHIJKLMNO
    // PQRSTUVWXYZ[\]^_
    // `abcdefghijklmno
    // pqrstuvwx

    // eFontStandard
    static const int codesStandard[] =
    {
        //      !     "     #     $     %     &     '     (     )     *     +     ,     -     .     /
        0x00, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
        //0     1     2     3     4     5     6     7     8     9     :     ;     <     =     >     ?
        0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
        //@     A     B     C     D     E     F     G     H     I     J     K     L     M     N     O
        0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F,
        //P     Q     R     S     T     U     V     W     X     Y     Z     [     \     ]     ^     _
        0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F,
        //`     a     b     c     d     e     f     g     h     i     j     k     l     m     n     o
        0x00, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E,
        //p     q     r     s     t     u     v     w     x
        0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77
    };

    // eFontScroll
    static const int codesScroll[] =
    {
        //      !     "     #     $     %     &     '     (     )     *     +     ,     -     .     /
        0x00, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x1D, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
        //0     1     2     3     4     5     6     7     8     9     :     ;     <     =     >     ?
        0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
        //@     A     B     C     D     E     F     G     H     I     J     K     L     M     N     O
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
        //P     Q     R     S     T     U     V     W     X     Y     Z     [     \     ]     ^     _
        0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F,
        //`     a     b     c     d     e     f     g     h     i     j     k     l     m     n     o
        0x00, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E,
        //p     q     r     s     t     u     v     w     x
        0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77
    };

    // eFontWalls
    static const int codesWalls[] =
    {
        //      !     "     #     $     %     &     '     (     )     *     +     ,     -     .     /
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x21, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1B, 0x1A,
        //0     1     2     3     4     5     6     7     8     9     :     ;     <     =     >     ?
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A,
        //@     A     B     C     D     E     F     G     H     I     J     K     L     M     N     O
        0x1A, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E,
        //P     Q     R     S     T     U     V     W     X     Y     Z     [     \     ]     ^     _
        0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A,
        //`     a     b     c     d     e     f     g     h     i     j     k     l     m     n     o
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A,
        //p     q     r     s     t     u     v     w     x
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A
    };

    const int* codes;
    int offset;
    QImage  file;

    if (font == eFontStandard)
    {
        codes = codesStandard;
        file = fileCache.getImage("gfx/interface/Font.png");
        offset = -2;
        pos.x -= 3;
    }
    else if (font == eFontScroll)
    {
        codes = codesScroll;
        file = fileCache.getImage("gfx/interface/Font.png");
        offset = -2;
        pos.x -= 3;
    }
    else
    {
        codes = codesWalls;
        file = fileCache.getImage("gfx/3DView/WallFont.png");
        offset = 0;
    }

    int i = 0;

    while (text[i] != 0)
    {
        int c = codes[text[i] - 32];

        CRect   rect(CVec2(c * 8, 0),
                     CVec2(c * 8 + 7, file.height() - 1));
        CVec2   cPos(pos.x + i * (8 + offset), pos.y);

        graph2D.drawImageAtlas(image, cPos, file, rect, (font != eFontWalls ? color : QColor(0, 0, 0, 0)));
        i++;
    }
}

//------------------------------------------------------------
bool   CInterface::isClickingOnArrow(SMouseArea* clickedArea, EMouseAreaTypes type)
{
    return (mouse.mButtonPressing && clickedArea != NULL && clickedArea->type == type);
}

//------------------------------------------------------------
void    CInterface::setDelayedArrow(EArrows arrow)
{
    delayedArrow = arrow;

    int maxOverload = 0;
    for (int i = 0; i < 4; ++i)
    {
        int overload = game.characters[i].overload();

        if (overload > maxOverload)
            maxOverload = overload;
    }

    if (maxOverload == 0)
        delayedArrowTime = OVERLOAD0_DELAY;
    else if (maxOverload == 1)
        delayedArrowTime = OVERLOAD1_DELAY;
    else
        delayedArrowTime = OVERLOAD2_DELAY;
}

//------------------------------------------------------------
void   CInterface::updateArrows(SMouseArea* clickedArea)
{
    if (delayedArrowTime == 0)
    {
        if (isClickingOnArrow(clickedArea, eMouseArea_ArrowForward) == true || keyboard.keys[CKeyboard::eForward] == true)
        {
            arrowHighlightTime = ARROWS_HIGHLIGHT_TIME;
            currentArrow = eArrowForward;
            setDelayedArrow(eArrowForward);
        }
        else if (isClickingOnArrow(clickedArea, eMouseArea_ArrowBackward) == true || keyboard.keys[CKeyboard::eBackward] == true)
        {
            arrowHighlightTime = ARROWS_HIGHLIGHT_TIME;
            currentArrow = eArrowBackward;
            setDelayedArrow(eArrowBackward);
        }
        else if (isClickingOnArrow(clickedArea, eMouseArea_ArrowLeft) == true || keyboard.keys[CKeyboard::eLeft] == true)
        {
            arrowHighlightTime = ARROWS_HIGHLIGHT_TIME;
            currentArrow = eArrowLeft;
            setDelayedArrow(eArrowLeft);
        }
        else if (isClickingOnArrow(clickedArea, eMouseArea_ArrowRight) == true || keyboard.keys[CKeyboard::eRight] == true)
        {
            arrowHighlightTime = ARROWS_HIGHLIGHT_TIME;
            currentArrow = eArrowRight;
            setDelayedArrow(eArrowRight);
        }
        else if (isClickingOnArrow(clickedArea, eMouseArea_ArrowTurnLeft) == true || keyboard.keys[CKeyboard::eTurnLeft] == true)
        {
            arrowHighlightTime = ARROWS_HIGHLIGHT_TIME;
            currentArrow = eArrowTurnLeft;
            player.turnLeft();
        }
        else if (isClickingOnArrow(clickedArea, eMouseArea_ArrowTurnRight) == true || keyboard.keys[CKeyboard::eTurnRight] == true)
        {
            arrowHighlightTime = ARROWS_HIGHLIGHT_TIME;
            currentArrow = eArrowTurnRight;
            player.turnRight();
        }
    }

    // mouvements retardés
    if (delayedArrowTime != 0)
    {
        delayedArrowTime--;

        if (delayedArrowTime == 0)
        {
            switch(delayedArrow)
            {
                case eArrowForward:
                    player.moveForward();
                    break;

                case eArrowBackward:
                    player.moveBackward();
                    break;

                case eArrowLeft:
                    player.moveLeft();
                    break;

                case eArrowRight:
                    player.moveRight();
                    break;

                default:
                    break;
            }
        }
    }
}

//------------------------------------------------------------

void   CInterface::update(SMouseArea* clickedArea)
{
    isPressingEye = false;

    if (clickedArea != NULL)
    {
        if (mouse.mButtonPressing == true)
        {
            switch (clickedArea->type)
            {
                // entrer dans l'inventaire
                case eMouseArea_Inventory:
                    mainState = eMainInventory;
                    currentChampion = (int)clickedArea->param1;
                break;

                // entrer dans la vue 3D
                case eMouseArea_3DView:
                {
                    if (isResurrecting == true)
                    {
                        for (int i = 3; i >= 0; --i)
                        {
                            if (game.characters[i].portrait != -1)
                            {
                                game.characters[i].portrait = -1;
                                break;
                            }
                        }
                    }
                    isResurrecting = false;
                    mainState = eMainGame;
                }
                break;

                // bouton ressusciter
                case eMouseArea_Resurrect:
                    game.lastMirrorClicked->setBoolParam("estVide", true);
                    isResurrecting = false;
                    mainState = eMainGame;
                break;

                // slot d'objet dans l'inventaire
                case eMouseArea_InvObject:
                {
                    // récupère le perso courant et la position dans l'inventaire
                    int champNum = (int)clickedArea->param1;
                    CCharacter* c = &game.characters[champNum];
                    int objectPos = (int)clickedArea->param2;

                    // échange l'objet que l'on tient avec celui dans l'inventaire
                    CObject temp = mouse.mObjectInHand;
                    mouse.mObjectInHand = c->bodyObjects[objectPos];
                    c->bodyObjects[objectPos] = temp;

                    // met à jour la charge du personnage
                    c->updateLoad();
                    c = &game.characters[selectedChampion];
                    c->updateLoad();
                }
                break;

                // clique sur le nom du champion
                case eMouseArea_SelectChamp:
                    selectedChampion = (int)clickedArea->param1;
                break;

                default:
                break;
            }
        }

        if (mouse.mButtonPressed == true)
        {
            if (clickedArea->type == eMouseArea_Eye)
                isPressingEye = true;
        }
    }

    updateArrows(clickedArea);
}

//------------------------------------------------------------
void CInterface::readTextDB()
{
    QDomDocument doc;

    QFile f("databases/texts.xml");
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement dbString = root.firstChild().toElement();

    while(!dbString.isNull())
    {
        if (dbString.tagName() == "text")
        {
            std::string id = dbString.attribute("id").toUtf8().constData();
            std::string text = dbString.text().toUtf8().constData();
            textsDB[id] = text;
        }
        dbString = dbString.nextSibling().toElement();
    }
}

//------------------------------------------------------------
std::string CInterface::getText(std::string id)
{
    return  textsDB[id];
}

//------------------------------------------------------------
void CInterface::enterResurrect(int num)
{
    mainState = eMainInventory;
    isResurrecting = true;
    currentChampion = num;
}
